@@ -11,14 +11,22 @@ def __status_name_filter(name):
 class BaseResponseClass(response.Response):
     """
     API response class with default status code.
+
+    :var status_code: HTTP status code.
+    :vartype status_code: int
+
+    :ivar timings: Response timings.
+    :vartype timings: int,None
+
+    :param timings: Response timings.
+    :vartype timings: int,None
     """
 
-    __slots__ = ('data',)
-
     def __init__(self, *args, **kwargs):
+        self.timings = kwargs.pop('timings', None)
         super().__init__(*args, **kwargs)
         if isinstance(self.data, str):
-            self.data = dict(detail=self.data)
+            self.data = {'detail': self.data}
 
 
 for __status_name in filter(__status_name_filter, dir(status)):
@@ -28,6 +36,6 @@ for __status_name in filter(__status_name_filter, dir(status)):
     __globals[__response_name] = type(
         __response_name,
         (BaseResponseClass,),
-        {"status_code": __status_code, "__slots__": ()}
+        {"status_code": __status_code}
     )
     __globals[__http_response_name] = __globals[__response_name]
