@@ -1,28 +1,39 @@
 # pylint: disable=no-member,redefined-outer-name,unused-argument
-from warnings import warn
-from collections import OrderedDict
-
-from django.urls.conf import include, re_path
+from django.urls.conf import include, re_path, path
 from django.conf import settings
+from django.core.exceptions import ImproperlyConfigured
+from django.utils.module_loading import import_string
 from rest_framework import routers, permissions, versioning
 
-from .base import Response
-from ..utils import import_class
+from . import responses
 
 
 class _AbstractRouter(routers.DefaultRouter):
 
     def __init__(self, *args, **kwargs):
-        self.custom_urls = list()
+        self.custom_urls = []
         self.permission_classes = kwargs.pop("perms", None)
-        self.create_schema = kwargs.pop('create_schema', False)
         super().__init__(*args, **kwargs)
 
+    def get_method_map(self, viewset, method_map):
+        # TODO: Remove it after 3.15 release with https://github.com/encode/django-rest-framework/pull/9057
+        bound_methods = {}
+        for method, action in method_map.items():
+            if getattr(viewset, action, None) is not None:
+                bound_methods[method] = action
+        return bound_methods
+
+    def _get_api_root_dict(self):
+        return {
+            reg[0]: self.routes[0].name.format(basename=reg[2])
+            for reg in self.registry
+        }
+
     def _get_custom_lists(self):
         return self.custom_urls
 
     def _get_views_custom_list(self, view_request, registers):
-        routers_list = OrderedDict()
+        routers_list = {}
         fpath = view_request.get_full_path().split("?")
         absolute_uri = view_request.build_absolute_uri(fpath[0])
         for prefix, _, name in self._get_custom_lists():
@@ -36,7 +47,7 @@ class _AbstractRouter(routers.DefaultRouter):
     def get_default_basename(self, viewset):
         base_name = getattr(viewset, 'base_name', None)
         if base_name is not None:
-            return base_name  # nocv
+            return base_name
         queryset = getattr(viewset, 'queryset', None)
         model = getattr(viewset, 'model', None)
         if queryset is None:  # nocv
@@ -75,57 +86,47 @@ class _AbstractRouter(routers.DefaultRouter):
 
     def generate(self, views_list):
         for prefix, options in views_list.items():
-            args = [prefix, import_class(options['view']), options.get('name', None)]
+            if 'view' in options:
+                view = import_string(options['view'])
+            elif 'model' in options:
+                view = import_string(options['model']).generated_view
+            args = [prefix, view]
+            if 'name' in options:
+                args.append(options['name'])
+            elif getattr(view, 'base_name', None) is None:
+                args.append(prefix)
             view_type = options.get('type', 'viewset')
             if view_type == 'viewset':
                 self.register(*args)
-            elif view_type == 'view':
+            elif view_type == 'view':  # nocv
                 self.register_view(*args)
             else:  # nocv
-                raise Exception('Unknown type of view')
+                raise ImproperlyConfigured(f'Unknown type of view for "{prefix}" with options: {options}')
 
 
 class APIRouter(_AbstractRouter):
-    root_view_name = 'v1'
+    default_root_view_name = 'v1'
 
     def __init__(self, *args, **kwargs):
+        self.router_version_name = kwargs.pop('version', self.default_root_view_name)
+        self.root_view_name = self.router_version_name
         super().__init__(*args, **kwargs)
-        if self.create_schema:
-            self.__register_schema()
         self.__register_openapi()
 
     def __register_openapi(self):
-        schema_view = import_class(settings.OPENAPI_VIEW_CLASS)
+        schema_view = import_string(settings.OPENAPI_VIEW_CLASS)
         cache_timeout = settings.SCHEMA_CACHE_TIMEOUT
-        swagger_kwargs = dict(
-            cache_timeout=0 if settings.DEBUG else cache_timeout,
-        )
+        swagger_kwargs = {'cache_timeout': 0 if settings.DEBUG or settings.TESTS_RUN else cache_timeout}
         self.register_view(
             '_openapi',
             schema_view.with_ui('swagger', **swagger_kwargs),
             name='_openapi'
         )
 
-    def __register_schema(self, name='schema'):
-        try:
-            self.register_view(rf'{name}', self._get_schema_view(), name)
-        except BaseException as exc:  # nocv
-            warn(f"Couldn't attach schema view: {exc}")
-
-    def _get_schema_view(self):
-        from rest_framework import schemas
-        return schemas.get_schema_view(
-            title=self.root_view_name,
-            version=self.root_view_name
-        )
-
     def get_api_root_view(self, *args, **kwargs):
-        list_name = self.routes[0].name
-        mapping = ((reg[0], list_name.format(basename=reg[2])) for reg in self.registry)
-        api_root_dict = OrderedDict(mapping)
 
         class API(self.APIRootView):
-            root_view_name = self.root_view_name
+            root_view_name = self.router_version_name
             if self.permission_classes:
                 permission_classes = self.permission_classes
             custom_urls = self.custom_urls
@@ -134,7 +135,7 @@ class APIRouter(_AbstractRouter):
                 # pylint: disable=invalid-name,no-self-argument
 
                 def determine_version(self_ver, request, *args, **kwargs):
-                    return self.root_view_name
+                    return self.router_version_name
 
             def get_view_name(self): return self.root_view_name
 
@@ -143,14 +144,15 @@ class APIRouter(_AbstractRouter):
                 data = self._get_views_custom_list(
                     request, super(API, self_inner).get(request, *args, **kwargs)
                 )
-                return Response(data, 200).resp
+                return responses.HTTP_200_OK(data)
 
-        return API.as_view(api_root_dict=api_root_dict)
+        return API.as_view(api_root_dict=self._get_api_root_dict())
 
     def get_urls(self):
         urls = super().get_urls()
-        for prefix, view, _ in self.custom_urls:  # nocv
-            view = view.as_view() if hasattr(view, 'as_view') else view
+        for prefix, view, _ in self.custom_urls:
+            # pylint: disable=cell-var-from-loop
+            view = getattr(view, 'as_view', lambda: view)()
             urls.append(re_path(f"^{prefix}/$", view))
         return urls
 
@@ -160,11 +162,9 @@ class APIRouter(_AbstractRouter):
                 'view': 'vstutils.api.views.LangViewSet'
             }
 
-        if r'_bulk' not in views_list:
-            views_list['_bulk'] = {
-                'view': 'vstutils.api.views.BulkViewSet',
-                'type': 'view',
-                'name': '_bulk'
+        if r'user' not in views_list:
+            views_list[r'user'] = {
+                'view': 'vstutils.api.auth.UserViewSet'
             }
 
         super().generate(views_list)
@@ -177,9 +177,6 @@ class MainRouter(_AbstractRouter):
         return super()._get_custom_lists() + self.routers
 
     def get_api_root_view(self, *args, **kwargs):
-        list_name = self.routes[0].name
-        mapping = ((reg[0], list_name.format(basename=reg[2])) for reg in self.registry)
-        api_root_dict = OrderedDict(mapping)
 
         class API(self.APIRootView):
             if self.permission_classes:
@@ -200,20 +197,20 @@ class MainRouter(_AbstractRouter):
                     'description': self_inner.view_name,
                     'current_version': links[settings.VST_API_VERSION],
                     'available_versions': {
-                        name: links[name]
-                        for name in links
+                        name: value
+                        for name, value in links.items()
                         if name in settings.API
                     }
                 }
-                for link in links:
+                for link, link_value in links.items():
                     if link not in data['available_versions']:
-                        data[link] = links[link]
-                return Response(data, 200).resp
+                        data[link] = link_value
+                return responses.HTTP_200_OK(data)
 
-        return API.as_view(api_root_dict=api_root_dict)
+        return API.as_view(api_root_dict=self._get_api_root_dict())
 
     def register_router(self, prefix, router, name=None):
-        name = name or router.root_view_name
+        name = name or getattr(router, 'router_version_name', router.root_view_name)
         self.routers.append((prefix, router, name))
 
     def unregister_router(self, prefix):
@@ -221,34 +218,31 @@ class MainRouter(_AbstractRouter):
 
     def get_urls(self):
         urls = super().get_urls()
-        for prefix, router, _ in self.routers:
-            urls.append(re_path(
+        for prefix, router, name in self.routers:
+            urls.append(path(
                 prefix,
                 include(
                     (router.urls, settings.VST_PROJECT),
-                    namespace=router.root_view_name
+                    namespace=getattr(router, 'router_version_name', name)
                 )
             ))
-        for prefix, view, _ in self.custom_urls:  # nocv
+        for prefix, view, name in self.custom_urls:
             # can't be tested because this initialization takes place before
             # any test code can be run
-            view = view.as_view() if hasattr(view, 'as_view') else view
-            urls.append(re_path(f"^{prefix}/$", view))
+            # pylint: disable=cell-var-from-loop
+            view = getattr(view, 'as_view', lambda: view)()
+            urls.append(re_path(f"^{prefix}/$", view, name=name))
         return urls
 
-    def generate_routers(self, api, create_schema=None, create_swagger=None):
+    def generate_routers(self, api):
         for version, views_list in api.items():
-            router = APIRouter(
-                perms=(permissions.IsAuthenticated,),
-                create_schema=create_schema or self.create_schema,
-            )
-            router.root_view_name = version
+            router = APIRouter(perms=(permissions.IsAuthenticated,), version=version)
             router.generate(views_list)
             self.register_router(version+'/', router)
 
         # Register view for new bulk requests
         self.register_view(
             r'endpoint',
-            import_class(settings.ENDPOINT_VIEW_CLASS),
+            import_string(settings.ENDPOINT_VIEW_CLASS),
             'endpoint'
         )
