@@ -1,7 +1,25 @@
+import os
+import time
+import copy
+
+import jwt
 from rest_framework import request as drf_request
 from django.conf import settings
+from django.utils.module_loading import import_string
 from drf_yasg import generators
 from drf_yasg.inspectors import field as field_insp
+from vstutils.utils import raise_context_decorator_with_default
+
+from .schema import get_nested_view_obj, _get_nested_view_and_subaction
+
+
+def get_centrifugo_public_address(request: drf_request.Request):
+    address = settings.CENTRIFUGO_PUBLIC_HOST
+    if address.startswith('/'):
+        address = request.build_absolute_uri(address)
+    elif 'api' in address.rsplit('/', 2):
+        address = address[:address.rfind('api')]
+    return os.path.join(address.replace('http', 'ws', 1), 'connection/websocket')
 
 
 class EndpointEnumerator(generators.EndpointEnumerator):
@@ -21,6 +39,18 @@ class VSTSchemaGenerator(generators.OpenAPISchemaGenerator):
         if not self.version:
             self.version = settings.VST_API_VERSION
 
+    def _get_hooks(self):
+        return map(
+            raise_context_decorator_with_default(),
+            filter(
+                bool,
+                map(
+                    raise_context_decorator_with_default()(import_string),
+                    getattr(settings, 'OPENAPI_HOOKS', ())
+                )
+            )
+        )
+
     def _get_subname(self, name):
         names = name.split('_')
         return ['_'.join(names[0:-1]), names[-1]]
@@ -39,8 +69,8 @@ class VSTSchemaGenerator(generators.OpenAPISchemaGenerator):
             return None  # nocv
         type_info.update({
             'description':
-                ('A unique {} value identifying '
-                 'instance of this {} sublist.').format(type_info['type'], query_name),
+                (f'A unique {type_info["type"]} value identifying '
+                 f'instance of this {query_name} sublist.'),
         })
         return type_info
 
@@ -72,9 +102,15 @@ class VSTSchemaGenerator(generators.OpenAPISchemaGenerator):
             if self._update_param_model(param, model, model_field):
                 continue
             elif self._update_param_view(param, model, view_cls):
-                continue
+                continue  # nocv
         return parameters
 
+    def should_include_endpoint(self, path, method, view, public):
+        nested_view, sub_action = _get_nested_view_and_subaction(view)
+        if nested_view and sub_action:
+            view = get_nested_view_obj(view, nested_view, sub_action, method)
+        return super().should_include_endpoint(path, method, view, public)
+
     def get_operation_keys(self, subpath, method, view):
         keys = super().get_operation_keys(subpath, method, view)
         subpath_keys = list(filter(bool, subpath.split('/')))
@@ -82,14 +118,35 @@ class VSTSchemaGenerator(generators.OpenAPISchemaGenerator):
         if method.upper() == 'GET' and '_detail' in r_type:
             keys = keys[:-1] + ['_'.join(r_type.split('_')[:-1])] + ['get']
         if r_type == 'get' and subpath_keys[-1] == gist:
-            if any([f for f in dir(view) if f.endswith('_'.join([gist, 'list']))]):
+            if any(f for f in dir(view) if f.endswith('_'.join([gist, 'list']))):
                 keys[-1] = 'list'
         return keys
 
-    def get_schema(self, request: drf_request.Request = None, *args, **kwargs):
+    def get_schema(self, request: drf_request.Request = None, *args, **kwargs):  # type: ignore
+        # pylint: disable=signature-differs
         if not getattr(request, 'version', ''):
-            request.version = self.version  # type: ignore
+            request.version = self.version
         result = super().get_schema(request, *args, **kwargs)
-        if request:
-            result['info']['x-user-id'] = request.user.id
+        if request and getattr(request, 'accepted_media_type', None) == 'application/openapi+json':
+            result['info']['x-user-id'] = request.user.pk
+            if (notificator := getattr(request, 'notificator', None)) and request.user.is_authenticated:
+                secret = notificator.get_openapi_secret()
+                if secret and request.user.pk:
+                    result['info']['x-centrifugo-token'] = jwt.encode(
+                        {
+                            "sub": request.session.session_key,
+                            "exp": int(time.time() + request.session.get_expiry_age()),
+                            "info": {
+                                'user_id': request.user.pk
+                            }
+                        },
+                        secret,
+                        algorithm="HS256"
+                    )
+                    result['info']['x-centrifugo-address'] = get_centrifugo_public_address(request)
+
+        for hook in self._get_hooks():
+            result = copy.deepcopy(result)
+            hook(request=request, schema=result)
+
         return result
