@@ -1,11 +1,16 @@
-# pylint: disable=django-not-available
+# pylint: disable=django-not-available,import-outside-toplevel
 import os
 import sys
+import functools
+
+import orjson
+import ormsgpack
+
 
 _default_settings = {
     # vstutils settings for generate settings
     "VST_PROJECT": "vstutils",
-    'VST_DEV_SETTINGS': 'settings.ini',
+    'VST_DEV_SETTINGS': 'dev_settings.ini',
     # django settings module
     "DJANGO_SETTINGS_MODULE": 'vstutils.settings',
     # Default urlconf
@@ -15,13 +20,13 @@ _default_settings = {
 
 def prepare_environment(default_settings=None, **kwargs):
     # pylint: disable=unused-argument
-    '''
+    """
     Prepare ENV for web-application
     :param default_settings: minimal needed settings for run app
     :type default_settings: dict
     :param kwargs: other overrided settings
     :rtype: None
-    '''
+    """
     default_settings = _default_settings if default_settings is None else default_settings
     for key, value in default_settings.items():
         os.environ.setdefault(key, value)
@@ -31,7 +36,7 @@ def prepare_environment(default_settings=None, **kwargs):
 def cmd_execution(*args, **kwargs):
     # pylint: disable=unused-variable
     """
-    Main function to executes from cmd. Emulates django-admin.py execution.
+    Main function to execute from cmd. Emulates django-admin.py execution.
     :param kwargs: overrided env-settings
     :rtype: None
     """
@@ -43,17 +48,40 @@ def cmd_execution(*args, **kwargs):
 
 
 def get_celery_app(name=None, **kwargs):  # nocv
-    # pylint: disable=import-error
-    '''
+    # pylint: disable=import-error,unused-import
+    """
     Function to return celery-app. Works only if celery installed.
     :param name: Application name
     :param kwargs: overrided env-settings
     :return: Celery-app object
-    '''
+    """
     from celery import Celery
+    from kombu.serialization import register as serialize_registrator
+    from .api.renderers import ORJSONRenderer, MsgpackRenderer
+    from . import drivers  # noqa: F401
+
     prepare_environment(**kwargs)
     name = name or os.getenv("VST_PROJECT")
-    celery_app = Celery(name)
+
+    # Override json encoder/decoder for tasks
+    json_encoder = functools.partial(ORJSONRenderer().render, media_type=ORJSONRenderer.media_type)
+    msgpack_encoder = functools.partial(MsgpackRenderer().render, accepted_media_type=MsgpackRenderer.media_type)
+    serialize_registrator(
+        'json',
+        encoder=json_encoder,
+        decoder=orjson.loads,
+        content_type=ORJSONRenderer.media_type,
+        content_encoding='binary',
+    )
+    serialize_registrator(
+        'msgpack',
+        encoder=msgpack_encoder,
+        decoder=ormsgpack.unpackb,
+        content_type='application/x-msgpack',
+        content_encoding='binary',
+    )
+
+    celery_app = Celery(name, task_cls='vstutils.tasks:TaskClass')
     celery_app.config_from_object('django.conf:settings', namespace='CELERY')
     celery_app.autodiscover_tasks()
     return celery_app
