@@ -1,12 +1,14 @@
-from __future__ import unicode_literals
 from typing import Text
 from collections import OrderedDict as odict
 import traceback
 import logging
 import json
+
 import ldap
 from django.conf import settings
 
+from .utils import translate
+
 
 def json_default(obj):  # nocv
     error_obj = TypeError(f"{type(obj)} is not JSON serializable")
@@ -14,8 +16,8 @@ def json_default(obj):  # nocv
         if isinstance(obj, bytes):
             return obj.decode("utf-8")
         raise error_obj
-    except:
-        raise error_obj
+    except Exception as err:
+        raise error_obj from err
 
 
 class LDAP:
@@ -27,8 +29,9 @@ class LDAP:
         'username',
         'password',
         'domain',
-        '__conn',
+        '_conn',
         'user_format',
+        'search_scope',
     )
     fields = ['cn', 'sAMAccountName', 'accountExpires', 'name', 'memberOf']
     LdapError = ldap.LDAPError
@@ -40,7 +43,7 @@ class LDAP:
         pass
 
     def __init__(self, connection_string: Text, username: Text = '', password: Text = None, domain: Text = None):
-        '''
+        """
         LDAP constructor
 
         :param connection_string: LDAP connection string ('ldap://server')
@@ -48,8 +51,9 @@ class LDAP:
                          or without but domain arg should be set.
         :param password: auth password
         :param domain: domain for easy use users
-        '''
+        """
         self.settings = settings
+        self.search_scope = getattr(ldap, f'SCOPE_{settings.LDAP_SEARCH_SCOPE}')
         self.user_format = settings.LDAP_FORMAT.replace('<', "{").replace('>', '}')
         self.logger = logging.getLogger(settings.VST_PROJECT_LIB)
         self.connection_string = connection_string
@@ -63,26 +67,30 @@ class LDAP:
         if not self.domain:
             if r'@' not in username:
                 raise self.InvalidDomainName(
-                    "Domain should be setuped or username should be with @domain.name"
+                    translate(
+                        "Domain should be setuped or username should be with @domain.name"
+                    )
                 )
         if len(self.domain.split('.')) <= 1:
             raise self.InvalidDomainName(
-                f"Invalid name {domain}. Should be [full.domain.name]"
+                translate("Invalid name") +
+                f" {domain}. " +
+                translate("Should be [full.domain.name].")
             )
         self.auth(self.username, self.password)
 
     def auth(self, username: Text = None, password: Text = None) -> None:
-        self.__conn = self.__authenticate(
+        self._conn = self.__authenticate(
             self.connection_string,
             str(username or self.username),
             str(password or self.password)
         )
 
     def __prepare_user_with_domain(self, username: Text) -> Text:
-        user = str(username).split('@')[0]
+        user = str(username).split('@', maxsplit=1)[0]
         domain = str(self.domain)
         if len(str(username).split('@')) > 1:
-            domain = str(username).split('@')[-1]
+            domain = str(username).split('@', maxsplit=1)[-1]
         domain = domain.lower()
         if domain != user:
             domain = ','.join([f'dc={d}' for d in domain.split('.') if d])
@@ -91,14 +99,14 @@ class LDAP:
         return user
 
     def __authenticate(self, ad: Text, username: Text, password: Text) -> ldap.functions.LDAPObject:
-        '''
+        """
         Active Directory auth function
 
         :param ad: LDAP connection string ('ldap://server')
         :param username: username with domain ('user@domain.name')
         :param password: auth password
         :return: ldap connection or None if error
-        '''
+        """
         result = None
         conn = ldap.initialize(ad)
         conn.protocol_version = 3
@@ -135,15 +143,15 @@ class LDAP:
         return self.__get_user_data()[1]
 
     def isAuth(self) -> bool:
-        '''
+        """
         Indicates that object auth worked
         :return: True or False
-        '''
-        if isinstance(self.__conn, ldap.ldapobject.LDAPObject) or self.__conn:
+        """
+        if isinstance(self._conn, ldap.ldapobject.LDAPObject) or self._conn:
             return True
         return False
 
-    def __ldap_filter(self, *filters):
+    def _ldap_filter(self, *filters):
         dc_list = [f"dc={i}" for i in self.domain_name.split('.') if i]
         additinal_filter = "".join([f"({i})" for i in filters if i])
         s_filter = f'(&(objectCategory=user){additinal_filter})'
@@ -151,14 +159,14 @@ class LDAP:
         self.logger.debug(
             f'Search in LDAP: {json.dumps(odict(BASE_DN=base_dn, FILTER=s_filter, FIELDS=self.fields))}'
         )
-        return base_dn, ldap.SCOPE_SUBTREE, s_filter, self.fields
+        return base_dn, self.search_scope, s_filter, self.fields
 
     def group_list(self, *args) -> Text:
         if not self.isAuth():
             raise self.NotAuth("Invalid auth.")
         try:
             data = {
-                k: v for k, v in self.__conn.search_s(*self.__ldap_filter(*args)) if k
+                k: v for k, v in self._conn.search_s(*self._ldap_filter(*args)) if k
             }
             return json.dumps(data, indent=4, ensure_ascii=False, default=json_default)
         except Exception:  # nocv
@@ -176,5 +184,5 @@ class LDAP:
         return f'[ {msg} {self.connection_string} -> {self.username} ]'
 
     def __del__(self):
-        if isinstance(getattr(self, '__conn', None), ldap.ldapobject.LDAPObject):
-            self.__conn.unbind_s()  # nocv
+        if isinstance(getattr(self, '_conn', None), ldap.ldapobject.LDAPObject):
+            self._conn.unbind_s()  # nocv
