@@ -1,5 +1,5 @@
 # pylint: disable=no-member
-from __future__ import unicode_literals
+import typing as _t
 import os
 import sys
 import time
@@ -7,7 +7,9 @@ from pathlib import Path
 from subprocess import CalledProcessError
 import subprocess
 import signal
+
 from django.conf import settings
+
 from ._base import BaseCommand
 from ...utils import raise_context
 
@@ -25,7 +27,7 @@ elif _uwsgi_default_path_alt3.exists():
     _uwsgi_default_path = _uwsgi_default_path_alt3
 
 
-def wait(proc, timeout=None, delay=0.1):
+def wait(proc, timeout=None, delay=0.01):
     while proc.poll() is None and (timeout or timeout is None):
         time.sleep(delay)
         if timeout is not None:
@@ -33,6 +35,38 @@ def wait(proc, timeout=None, delay=0.1):
     return proc.poll()
 
 
+def get_celery_command(celery_path='celery', command='worker', **kwargs):
+    # Format args string.
+    options = ''
+    app_option = f'--app={settings.VST_PROJECT}.wapp:app'
+    args = kwargs.pop('_args', ())
+    options += ' '.join(filter(bool, args))
+    options += ' '
+
+    if command == 'worker':
+        options_dict = {**settings.WORKER_OPTIONS, **kwargs}
+    else:
+        options_dict = kwargs
+
+    for key, value in options_dict.items():
+        if key == 'app':
+            app_option = "--app={}".format(value.replace(',', r'\,'))
+            continue
+        is_boolean = isinstance(value, bool)
+        if (is_boolean and value) or value:
+            options += f' -{"-" if key != "O" else ""}{key}'
+        if is_boolean:
+            continue
+        options += "{}{}".format('=' if key != 'O' else '', value.replace(',', r'\,'))
+
+    # Add queues list to celery args
+    if '--queues' not in options and command == 'worker':
+        options += ' --queues={}'.format(r'\,'.join(settings.WORKER_QUEUES))
+
+    # Add arguments to uwsgi cmd list.
+    return f'{celery_path} {app_option} {command} {options}'
+
+
 class Command(BaseCommand):
     help = "Backend web-server."
     _uwsgi_default_path = _uwsgi_default_path
@@ -79,39 +113,20 @@ class Command(BaseCommand):
         return args
 
     def _get_worker_options(self):
-        cmd = []
+        cmd: _t.List[str] = []
 
         # Check that worker is enabled in settings.
-        if not settings.RUN_WORKER:
+        if not getattr(settings, 'RUN_WORKER', False):
             return cmd
 
-        # Get celery args from settings.
-        worker_options = settings.WORKER_OPTIONS
-
-        # Format args string.
-        options = ''
-        for key, value in worker_options.items():
-            is_boolean = isinstance(value, bool)
-            if (is_boolean and value) or value:
-                options += f' --{key}'
-            if is_boolean:
-                continue
-            options += "={}".format(value.replace(',', r'\,'))
-
-        # Add queues list to celery args
-        if '--queues' not in options:
-            options += ' --queues={}'.format(r'\,'.join(settings.WORKER_QUEUES))
-
         # Add arguments to uwsgi cmd list.
         cmd.append('--attach-daemon2')
-        run = 'stopsignal=15,reloadsignal=1,'
-        run += 'exec=celery worker'
-        run += options
-        cmd.append(run)
+        cmd.append(f'stopsignal=15,reloadsignal=1,exec={get_celery_command()}')
 
         return cmd
 
     def handle(self, *uwsgi_args, **opts):
+        # pylint: disable=too-many-branches
         super().handle(*uwsgi_args, **opts)
 
         # Environment
@@ -122,14 +137,12 @@ class Command(BaseCommand):
             str(opts['script']),
             f'--set-ph=program_name={settings.VST_PROJECT}',
             f'--set-ph=lib_name={settings.VST_PROJECT_LIB}',
-            f'--set-ph=asgi_app={settings.ASGI_APPLICATION}',
             f'--set-ph=api_path={settings.API_URL}',
-            f'--set-ph=ws={"true" if settings.HAS_CHANNELS else "false"}',
-            f'--module={settings.UWSGI_APPLICATION}',
+            f'--set-ph=vstutils_version={settings.VSTUTILS_VERSION}',
+            f'--env=DJANGO_SETTINGS_MODULE={os.getenv("DJANGO_SETTINGS_MODULE")}',
+            f'--python-worker-override={settings.UWSGI_WORKER_PATH}',
+            f'--socket={opts["addrport"]}'
         ]
-        #  Setup http addr:port.
-        addrport = opts['addrport']
-        cmd += [f'--{"https" if len(addrport.split(",")) > 1 else "http"}={addrport}']
 
         # Import uwsgi-args from this command args (key=value).
         cmd += [
@@ -140,17 +153,15 @@ class Command(BaseCommand):
         if opts['config'] and Path(opts['config']).exists():
             cmd += [f'--ini={opts["config"]}']
 
-        # Connect static files.
-        for static_path in settings.STATIC_FILES_FOLDERS:
-            if f"/static={static_path}" not in cmd:
-                cmd += ['--static-map', f"/static={static_path}"]
-
         # Append uwsgi configs.
-        for cf in settings.VST_PROJECT_LIB, settings.VST_PROJECT:
-            config_file = Path('/etc/') / Path(cf) / 'settings.ini'
-            option = f'--ini={str(config_file)}'
-            if config_file.exists() and option not in cmd:
+        for config_file in map(Path, settings.CONFIG_FILES):
+            option = f'--{"ini" if config_file.suffix == ".ini" else "yml"}={str(config_file)}'
+            if not (config_file.exists() and option not in cmd):
+                continue
+            if config_file.suffix == '.ini':
                 cmd.append(option)
+            elif config_file.suffix == '.yml':
+                cmd.append(f'--yml={str(config_file)}')
 
         if self.uwsgi_default_config.exists():
             cmd.append(str(self.uwsgi_default_config))
@@ -177,16 +188,16 @@ class Command(BaseCommand):
         # Run web server
         try:
             self._print('Execute: ' + ' '.join(cmd))
-            proc = subprocess.Popen(cmd, env=env, stdin=read)
-            try:
-                wait(proc)
-            except BaseException as exc:
-                proc.send_signal(signal.SIGTERM)
-                wait(proc, 10)
-                with raise_context():
-                    proc.kill()
-                wait(proc)
-                raise exc
+            with subprocess.Popen(cmd, env=env, stdin=read) as proc:
+                try:
+                    wait(proc)
+                except BaseException as exc:
+                    proc.send_signal(signal.SIGTERM)
+                    wait(proc, 10)
+                    with raise_context():
+                        proc.kill()
+                    wait(proc)
+                    raise exc
         except KeyboardInterrupt:
             self._print('Exit by user...', 'WARNING')
         except CalledProcessError as err:
